package de.fzj.unicore.ucc.actions;

import org.apache.commons.cli.OptionBuilder;
import org.unigrids.services.atomic.types.ProtocolType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.ucc.MessageWriter;
import de.fzj.unicore.ucc.helpers.Location;
import de.fzj.unicore.ucc.util.FileDownloader;
import de.fzj.unicore.ucc.util.Mode;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;
import de.fzj.unicore.wsrflite.xmlbeans.client.IRegistryQuery;
import eu.unicore.security.util.client.IClientProperties;

/**
 * get a file from a remote sms
 */
public class GetFile extends FileOperation {

	protected Location sourceDesc;
	protected StorageClient sms;
	protected boolean append;
	protected boolean recurse;
	
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_APPEND_LONG)
				.withDescription("Append to (existing) file(s)")
				.isRequired(false)
				.create(OPT_APPEND)
		);
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_RECURSIVE_LONG)
				.withDescription("Recurse into subdirectories")
				.isRequired(false)
				.create(OPT_RECURSIVE)
		);
	}
	
	@Override
	public void process() {
		super.process();
		sourceDesc=resolve(source,registry,securityProperties,this);
		verbose("Source file URL "+sourceDesc.getUnicore6URI());
		append=getBooleanOption(OPT_APPEND_LONG, OPT_APPEND);
		verbose("Appending data="+append);
		recurse=getBooleanOption(OPT_RECURSIVE_LONG, OPT_RECURSIVE);
		if(recurse)verbose("Recurse into subdirectories="+recurse);
		try{
			EndpointReferenceType e=EndpointReferenceType.Factory.newInstance();
			e.addNewAddress().setStringValue(sourceDesc.getSmsEpr());
			String dn=findServerName(e);
			if(dn!=null)WSUtilities.addServerIdentity(e, dn);
			sms=new StorageClient(sourceDesc.getSmsEpr(),e,securityProperties);
			verbose("SMS contacted, server reply: servertime="+sms.getCurrentTime());
			getFile();
		}catch(Exception e){
			error("Can't get file.",e);
			endProcessing(ERROR);
		}
	}
	
	/**
	 * resolve a location
	 * 
	 * @param url
	 * @param registry
	 * @param msg
	 * @return
	 */
	protected Location resolve(String url, IRegistryQuery registry, IClientProperties sec,MessageWriter msg){
	    return Resolve.resolve(url,registry,sec,msg);
	}
	
	protected void getFile()throws Exception{
		Mode mode=append?Mode.append:Mode.overwrite;
		FileDownloader exp=new FileDownloader(sourceDesc.getName(),target,mode);
		String sourceDescProtocol=sourceDesc.getProtocol();
		if(!"BFT".equalsIgnoreCase(sourceDescProtocol)){
			try{
				preferredProtocols.add(0, ProtocolType.Enum.forString(sourceDescProtocol));
			}catch(Exception ignored){}
		}
		exp.setPreferredProtocols(preferredProtocols);
		exp.setExtraParameterSource(properties);
		exp.setTiming(timing);
		exp.setRecurse(recurse);
		exp.perform(sms, this);
	}

	@Override
	public String getName() {
		return "get-file";
	}

	@Override
	public String getSynopsis() {
		return "Gets a file from remote location '-s' and writes it to the local file specified by the '-t' option.";
	}
	@Override
	public String getDescription(){
		return "get remote files";
	}

	@Override
	public String getArgumentList(){
		return "";
	}
	@Override
	public String getCommandGroup(){
		return "Data management";
	}
	
}
