package de.fzj.unicore.ucc.shell;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Collections;
import java.util.List;

import jline.Completor;
import de.fzj.unicore.ucc.Command;
import de.fzj.unicore.ucc.UCC;

/**
 * provide basic command completion for the "shell" command
 */
public class UCCCompletor implements Completor{

	final String[] cmds;

	UCCCompletor(String[] cmds){
		this.cmds=cmds;
	}

	@SuppressWarnings("rawtypes")
	public int complete(String buffer, int cursor, List completions) {
		String cmd=buffer.trim();
		completions.clear();

		String[] parts=buffer.split(" +");
		if(cursor<=parts[0].length()){
			//complete UCC command name
			return completeUCCCommand(parts, completions);
		}
		else {
			Command command=UCC.getCommand(parts[0]);
			return completeCommand(command, cmd, cursor, completions);
		}
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public int completeUCCCommand(String[] parts, List completions) {
		String cmd=parts[0];
		completions.clear();
		for(String c: cmds){
			if (c.startsWith(cmd))completions.add(c);
		}
		Collections.sort(completions);
		return 0;
	}

	//TODO
	@SuppressWarnings("rawtypes")
	protected int completeCommand(Command command, String buffer, int cursor, List completions){
		try{
			//get the last part of user input and try to complete it
			//as a filename
			String[] parts=buffer.split(" +");
			String last=parts[parts.length-1];

			String base=buffer.substring(0, buffer.lastIndexOf(last));
			return completeFileName(base,last,cursor,completions);
		}catch(Exception ex){/*ignore*/}
		return -1;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	protected int completeFileName(String base, String filePattern,int cursor, List completions){
		try{
			File f=new File(filePattern);
			final String fileName=f.getName();
			final boolean isDirectory=f.isDirectory();
			File parent=isDirectory?f:f.getParentFile();
			for(File child: parent.listFiles(new FilenameFilter(){
				public boolean accept(File dir, String name) {
					return isDirectory || name.startsWith(fileName);
				}
			})){
				String completion=f.isAbsolute()?child.getAbsolutePath():child.getPath();
				completions.add(base+completion);
			}

		}catch(Exception ex){/*ignore*/}
		return 0;
	}

}
