package de.fzj.unicore.ucc.actions;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import org.junit.Test;
import org.oasisOpen.docs.wsrf.rp2.GetResourcePropertyDocumentResponseDocument;
import org.unigrids.x2006.x04.services.jms.JobPropertiesDocument;

import de.fzj.unicore.ucc.UCC;
import de.fzj.unicore.ucc.util.EmbeddedTestBase;
import eu.unicore.bugsreporter.annotation.FunctionalTest;

/**
 * Functional tests for the UCC actions. 
 * These run against an embedded UNICORE instance.
 */
public class TestJobRelatedActions extends EmbeddedTestBase {
		
	@FunctionalTest(id="http://testRunSimpleJob", description="Connects to the test server and runs a simple job")
	@Test
	public void test_Run(){
		connect();
		runDate();
	}

	@FunctionalTest(id="http://testRunJobWithUploads", description="Runs a job with file uploads")
	@Test
	public void test_Run_JobWithUploads(){
		connect();
		run("src/test/resources/jobs/date-with-uploads.u", false);
	}

	@FunctionalTest(id="http://testRunJobWithU6Staging", description="Runs a job with file staging from/to u6")
	@Test
	public void test_Run_WithU6Staging(){
		connect();
		upload("src/test/resources/testfiles/file1", "u6://UCC-Test/WORK/file1", true);
		run("src/test/resources/jobs/date-with-u6-staging.u", true);
	}
	
	@FunctionalTest(id="http://testRunJobWithErrorInStageOut", description="Runs a job using the failonerror flag")
	@Test
	public void test_Run_WithErrorInStageOut(){
		connect();
		run("src/test/resources/jobs/date-with-error-in-staging.u", true);
	}
	
	@FunctionalTest(id="http://testRunJobWithWildcardExports", description="Runs a job and export results using wildcards")
	@Test
	public void test_Run_JobWithWildcardExports(){
		connect();
		run("src/test/resources/jobs/date-with-wildcard-exports.u", false);
	}
	
	@FunctionalTest(id="testListJobs", description="Tests list-job command")
	@Test
	public void test_Run_and_ListJobs(){
		connect();
		runDate();
		
		String[] args=new String[]{"list-jobs", "-v", 
				"-c", "src/test/resources/conf/userprefs.embedded",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
	}

	@FunctionalTest(id="testRunJSDL", description="Runs a job defined in a JSDL file")
	@Test
	public void test_Run_JSDL(){
		connect();
		String[] args=new String[]{"run",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"-j","src/test/resources/jobs/date.jsdl",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
	}

	@FunctionalTest(id="testRunAsync", description="Runs a job in async mode")
	@Test
	public void test_Run_Async(){
		connect();

		String[] args=new String[]{"run", "-v",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"src/test/resources/jobs/date.u",
				"-a"
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);

		String id=Run.getLastJobAddress();
		int c=0;
		do{

			args=new String[]{"get-status", "-v",
					"-c", "src/test/resources/conf/userprefs.embedded",
					id,
			};
			UCC.main(args);
			assertEquals(Integer.valueOf(0),UCC.exitCode);
			c++;
			
		}while(c<5 && ! GetStatus.lastStatus.contains("SUCCESSFUL"));


		args=new String[]{"get-output", "-v",
				"-c", "src/test/resources/conf/userprefs.embedded",
				id,
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);

	}

	@FunctionalTest(id="testAbort", description="Tests the abort command")
	@Test
	public void test_Run_and_Abort(){
		connect();

		String[] args=new String[]{"run", "-v",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"src/test/resources/jobs/sleep.u",
				"-a"
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);

		
		args=new String[]{"abort-job", "-v", 
				"-c", "src/test/resources/conf/userprefs.embedded",
				Run.getLastJobAddress(),
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
	}
	
	protected String createNewUspace(){
		String[] args=new String[]{"run",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"src/test/resources/jobs/date.u",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		
		args=new String[]{"wsrf", "getproperties", 
				"-c", "src/test/resources/conf/userprefs.embedded",
				Run.getLastJobAddress(),
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		
		try{
			GetResourcePropertyDocumentResponseDocument d=GetResourcePropertyDocumentResponseDocument.Factory.parse(WSRF.getLastProperties());
			JobPropertiesDocument jd= JobPropertiesDocument.Factory.parse(d.getGetResourcePropertyDocumentResponse().newInputStream());
			return jd.getJobProperties().getWorkingDirectoryReference().getAddress().getStringValue();
		}catch(Exception ex){
			ex.printStackTrace();
			fail(); 
		}
		return null;
	}

}
