package de.fzj.unicore.ucc.shell;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

public class TestUCCCompletor {
	
	@Test
	public void testCompleteCommand(){
		String[]cmd={"foo","bar","ham"};
		List<String>completions=new ArrayList<String>();
		new UCCCompletor(cmd).complete("f", 0, completions);
		assertEquals(1,completions.size());
		new UCCCompletor(cmd).complete("ba", 1, completions);
		assertEquals(1,completions.size());
		assertEquals("bar",completions.get(0));
	}
	
	@Test
	public void testFileNameCompletion(){
		String[]cmd={"foo","bar","ham"};
		String filePattern="src/test/resources/uas/sample";
		List<String>completions=new ArrayList<String>();
		new UCCCompletor(cmd).completeFileName("",filePattern, 1, completions);
		assertEquals(1,completions.size());
		assertEquals("src/test/resources/uas/sample_policy.xml",completions.get(0));
	}

	@Test
	public void testFileNameCompletionAbsolute(){
		File base=new File("src/test/resources/uas/");
		File file=new File("src/test/resources/uas/xa");
		String filePattern=file.getAbsolutePath();
		String[]cmd={"foo","bar","ham"};
		List<String>completions=new ArrayList<String>();
		new UCCCompletor(cmd).completeFileName("",filePattern, 1, completions);
		assertEquals(1,completions.size());
		String expect=new File(base,"xacml.config").getAbsolutePath();
		assertEquals(expect,completions.get(0));
	}
	
	@Test
	public void testCompleteDirectoryNames(){
		File base=new File("src/test/resources/");
		String filePattern=base.getPath();
		int num=base.list().length;
		List<String>completions=new ArrayList<String>();
		String[]cmd={"foo","bar","ham"};
		new UCCCompletor(cmd).completeFileName("",filePattern, 1, completions);
		assertEquals(num,completions.size());
	}
	
	
}
